// Bully Box V2
// Caspar Gelderman
// 9 augustus 2021

#include <Servo.h>

#define ECHO 2
#define TRIGGER 3

#define BUTTON 8
#define BUZZER 12

// Button
int state = LOW;                  // Saved state of the button
int reading;                      // Save the current state of the button
int previous = HIGH;              // Save the previous state of the button

unsigned long buttonTime = 0;     // Keep track of the current time since last button debounce
long debounce = 200;              // minimum time between detecting button presses to make it more reliable

bool toggledOn = false;

// Sensor
unsigned long duration = 0;       // Save the duration for sound wave to return
int distance;                     // Save the distance calculated by the sensor
unsigned long sensorTime = 0;     // Keep track of the current time since last signal
long OnTime = 10;                 // Amount of time where the sensor is turned on in microseconds
long OffTime = 2;                 // Amount of time between signals in microseconds
int triggerState = LOW;           // State of the Sensor state

// Servo
Servo servo;

int triggerDistance = 10; // max distance of the unfortunate finger above the sensor for the servo to trigger

void setup() {
  Serial.begin(9600);

  pinMode(TRIGGER, OUTPUT);
  pinMode(ECHO, INPUT);

  pinMode(BUTTON, INPUT);
  pinMode(BUZZER, OUTPUT);

  servo.attach(7);
}

void loop() {
  // turn the box "on" or "off"
  buttonToggle();

  // only calculate distance when the box is turned "on"
  if (toggledOn) {
    sensor();
  }

  buzzer();
  moveServo();
}


// Toggle the button
void buttonToggle() {
  reading = digitalRead(BUTTON);

  // The timer is to make sure the button registers more reliably
  // (NOTE: using a setup like below; a timer that fires once every specified duration 
  //    acts like the delay() function, but while delay() pauses the whole system this does not)
  if (reading == HIGH && previous == LOW && millis() - buttonTime > debounce) {
    state = !state;
    toggledOn = state;

    if (toggledOn) delay(1000);   // pause everything for one second after the box is toggled on 
                                      //so you won't get your finger chopped off immediately
    buttonTime = millis();        // Set the last time since button check to the current time
  }

  previous = reading;   // Set the last state of the button to the current state of the button
}

// play a nice, definitely not horrible tone
void buzzer() {
  //Only play the tone when the box is toggled on, and immediately stop the tone from playing when it is turned off
  if (toggledOn) {
    digitalWrite(12, HIGH);
    tone(BUZZER, 1000);
  } else {
    noTone(BUZZER);
    digitalWrite(12, LOW);
  }
}

// Calculate the distance from the sensor
void sensor() {
  if ((triggerState == LOW) && (micros() - sensorTime >= OffTime))
  {
    triggerState = HIGH;
    sensorTime = micros();
    digitalWrite(TRIGGER, triggerState);
  }
  else if ((triggerState == HIGH) && (micros() - sensorTime >= OnTime))
  {
    triggerState = LOW;
    sensorTime = micros();
    digitalWrite(TRIGGER, triggerState);
  }
  duration = pulseIn(ECHO, HIGH);
  // 0.017 is the value that represents the speed of sound divided by 2 to calculate distance
  distance = duration * 0.017;
}

// Trigger the servo whenever the object is within the specified distance
void moveServo() {
  if ((distance <= triggerDistance && distance > 0) && toggledOn) {
    servo.write(90);
  } else {
    servo.write(0);
  }
}
